<?php
/**
 * Main Plugin
 *
 * @package   dorzki\WooCommerce\BOGO
 * @author    Dor Zuberi <admin@dorzki.io>
 * @copyright 2023 dorzki
 * @version   1.0.0
 */

namespace dorzki\WooCommerce\BOGO;

use Throwable;
use WC_Cart;
use WC_Product;


/**
 * Class Plugin
 *
 * @package dorzki\WooCommerce\BOGO
 */
class Plugin {
	const META_FIELD = '_wc_bogo_product';


	/**
	 * Plugin constructor.
	 */
	public function __construct() {
		add_action( 'woocommerce_product_options_related', [ $this, 'add_bogo_field_to_linked_products_section' ] );
		add_action( 'woocommerce_process_product_meta', [ $this, 'save_bogo_field_selection' ] );

		add_action( 'woocommerce_before_calculate_totals', [ $this, 'override_bogo_product_price' ], 10, 2 );
		add_action( 'woocommerce_cart_item_removed', [ $this, 'maybe_remove_bogo_product' ], 10, 2 );

		add_filter( 'woocommerce_add_to_cart_validation', [ $this, 'maybe_add_bogo_product' ], 10, 2 );

		add_filter( 'woocommerce_short_description', [ $this, 'maybe_display_promotion_notice' ] );
	}


	/**
	 * Add new BOGO Promotion product selection field to product "linked products" section.
	 *
	 * @return void
	 */
	public function add_bogo_field_to_linked_products_section() {
		global $post;

		$bogo_product_id = get_post_meta( $post->ID, self::META_FIELD, true );

		echo '<div class="options_group">';
		echo '  <p class="form-field">';
		echo '		<label for="upsell_ids">' . esc_html__( 'BOGO Promotion', 'dorzki-woocommerce-bogo' ) . '</label>';
		echo '      <select class="wc-product-search" style="width: 50%;" id="bogo_promotion" name="bogo_promotion" data-placeholder="' . esc_attr__( 'Search for a product&hellip;', 'dorzki-woocommerce-bogo' ) . '" data-action="woocommerce_json_search_products" data-exclude="' . $post->ID . '">';

		if ( $bogo_product_id ) {
			$bogo_product = wc_get_product( $bogo_product_id );

			if ( is_object( $bogo_product ) ) {
				echo '<option value="' . esc_attr( $bogo_product->get_id() ) . '"' . selected( true, true, false ) . '>' . esc_html( wp_strip_all_tags( $bogo_product->get_formatted_name() ) ) . '</option>';
			}
		}

		echo '      </select>';
		echo '  </p>';
		echo '</div>';
	}

	/**
	 * Save BOGO Promotion selected product id.
	 *
	 * @param int $post_id Product id
	 *
	 * @return void
	 */
	public function save_bogo_field_selection( int $post_id ) {
		$bogo_product = ! empty( $_POST['bogo_promotion'] ) ? intval( $_POST['bogo_promotion'] ) : null;

		update_post_meta( $post_id, self::META_FIELD, $bogo_product );
	}


	/**
	 * Override BOGO Promotion product price.
	 *
	 * @param WC_Cart $wc_cart WooCommerce cart
	 *
	 * @return void
	 */
	public function override_bogo_product_price( WC_Cart $wc_cart ) {
		foreach ( $wc_cart->get_cart_contents() as $cart_item ) {
			if ( isset( $cart_item['is_bogo_promotion'] ) && true === $cart_item['is_bogo_promotion'] ) {
				/** @var WC_Product $product */
				$product = $cart_item['data'];
				$product->set_price( 0 );
				$product->set_sold_individually( true );
			}
		}
	}

	/**
	 * Maybe remove BOGO Promotion product from cart.
	 *
	 * @param string $cart_item_key Removed product cart item key
	 * @param WC_Cart $wc_cart WooCommerce cart
	 *
	 * @return void
	 */
	public function maybe_remove_bogo_product( string $cart_item_key, WC_Cart $wc_cart ) {
		if ( is_admin() ) {
			return;
		}

		$parent_product_id = $wc_cart->get_removed_cart_contents()[ $cart_item_key ]['product_id'];

		$bogo_product_id = get_post_meta( $parent_product_id, self::META_FIELD, true );

		if ( ! empty( $bogo_product_id ) ) {
			$bogo_product_cart_id       = $wc_cart->generate_cart_id( $bogo_product_id, 0, [], [
				'is_bogo_promotion'      => true,
				'bogo_linked_product_id' => $parent_product_id
			] );
			$bogo_product_cart_item_key = $wc_cart->find_product_in_cart( $bogo_product_cart_id );

			if ( ! empty( $bogo_product_cart_item_key ) ) {
				$wc_cart->remove_cart_item( $bogo_product_cart_item_key );
			}
		}
	}


	/**
	 * Check whether to add product BOGO promotion.
	 *
	 * @param bool $passed Validation passed?
	 * @param int $product_id Product id
	 *
	 * @return bool
	 */
	public function maybe_add_bogo_product( bool $passed, int $product_id ): bool {
		if ( is_admin() ) {
			return $passed;
		}

		$bogo_product_id = get_post_meta( $product_id, self::META_FIELD, true );

		if ( empty( $bogo_product_id ) ) {
			return $passed;
		}

		try {
			WC()->cart->add_to_cart( $bogo_product_id, 1, 0, [], [
				'is_bogo_promotion'      => true,
				'bogo_linked_product_id' => $product_id
			] );
		} catch ( Throwable $ex ) {
			return false;
		}

		return $passed;
	}


	/**
	 * Maybe display a BOGO promotion notice in product page.
	 *
	 * @param string $description Product short description
	 *
	 * @return string
	 */
	public function maybe_display_promotion_notice( string $description ): string {
		global $post;

		if ( ! is_product() ) {
			return $description;
		}

		$bogo_product_id = get_post_meta( $post->ID, self::META_FIELD, true );

		if ( empty( $bogo_product_id ) ) {
			return $description;
		}

		$bogo_product = wc_get_product( $bogo_product_id );
		$product_link = sprintf(
			'<a href="%s" target="_blank">%s</a>',
			$bogo_product->get_permalink(),
			$bogo_product->get_name()
		);

		return wc_kses_notice(
			       '<div class="woocommerce-message woocommerce-message--info">' .
			       sprintf(
				       esc_html__( '🎁 Buy this product and get %s for free!', 'dorzki-woocommerce-bogo' ),
				       $product_link
			       ) .
			       '</div>'
		       ) . $description;
	}
}


// Init class.
new Plugin();
